--- fam/Scheduler.c++.orig	Tue Jul 16 07:23:25 2002
+++ fam/Scheduler.c++	Fri Sep  6 23:22:54 2002
@@ -1,3 +1,4 @@
+//  Copyright (C) 2000 David Rufino <daverufino@btinternet.com>
 //  Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 //  
 //  This program is free software; you can redistribute it and/or modify it
@@ -26,12 +27,15 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#ifdef HAVE_KQUEUE
 #include <stdlib.h>
+#endif
 #include <sys/param.h>
 
 #include "Log.h"
 #include "timeval.h"
 
+#ifndef HAVE_KQUEUE
 //  Define a bunch of class-global variables.
 
 Scheduler::IOTypeInfo	 Scheduler::read(&FDInfo::read);
@@ -347,3 +351,288 @@
 
     do_tasks();
 }
+#else /* HAVE_KQUEUE */
+
+bool				Scheduler::running;
+int				Scheduler::kq;
+struct kevent			Scheduler::kevents[50];
+std::list<struct kevent>		Scheduler::my_list;
+struct kevent			Scheduler::new_events[10];
+int				Scheduler::new_cnt;
+
+int				Scheduler::ntasks;
+timeval				Scheduler::next_task_time;
+Scheduler::onetime_task *	Scheduler::first_task;
+Scheduler::TimedProc		Scheduler::recurring_proc;
+void *				Scheduler::recurring_closure;
+timeval				Scheduler::recurring_interval;
+
+struct kevent *
+Scheduler::get_new_event (void)
+{
+	assert (new_cnt < 10);
+
+	return (&new_events[new_cnt++]);
+}
+
+Scheduler::IOHandler
+Scheduler::remove_handler (int fd, int filter)
+{
+	struct kevent		*kev;
+	std::list<struct kevent>::iterator old;
+	IOHandler		handler;
+
+	old = kev_find (fd, filter);
+	assert (old != my_list.end() && old->ident == fd);
+
+	handler = ((handler_data *)old->udata)->handler;
+	delete old->udata;
+
+	kev = get_new_event ();
+	*kev = *old;
+	kev->flags = EV_DELETE|EV_DISABLE;
+
+	my_list.erase (old);
+
+	return handler; 
+}
+
+std::list<struct kevent>::iterator
+Scheduler::kev_find (int fd, int filter)
+{
+	std::list<struct kevent>::iterator	iter;
+
+	for (iter = my_list.begin(); iter != my_list.end(); iter++) {
+		if (iter->ident == fd && iter->filter == filter)
+			return iter;
+	}
+
+	return my_list.end();
+}
+
+Scheduler::IOHandler
+Scheduler::install_handler (int fd, IOHandler handler, void *closure, int filter)
+{
+	struct kevent		*kev;
+	std::list<struct kevent>::iterator old;
+	IOHandler		old_handler = NULL; 
+	handler_data		*data;
+
+	assert (fd >= 0);
+
+	old = kev_find (fd, filter);
+	if (old != my_list.end()) {
+		data = (handler_data *)old->udata;
+		old_handler = data->handler;
+		delete (handler_data *)data;
+	}
+
+/*
+Re-adding an existing event will modify the parameters of the original event, and not result in a duplicate entry.
+ */
+	kev = get_new_event();
+	data = new handler_data;
+	data->handler = handler;
+	data->closure = closure;
+
+	kev->ident = fd;
+	kev->filter = filter;
+	kev->flags = EV_ADD|EV_CLEAR|EV_ENABLE;
+	kev->fflags = 0;
+	kev->data = 0;
+	kev->udata = (void *)data;
+
+	my_list.push_back (*kev);
+
+	return (old_handler);
+}
+
+Scheduler::IOHandler
+Scheduler::install_read_handler (int fd, IOHandler handler, void *closure)
+{
+	Log::log (Log::DEBUG, "install_read_handler (%d)", fd);
+	return Scheduler::install_handler (fd, handler, closure, EVFILT_READ);
+}
+
+Scheduler::IOHandler
+Scheduler::remove_read_handler (int fd)
+{
+	Log::log (Log::DEBUG, "remove_read_handler (%d)", fd);
+	return Scheduler::remove_handler (fd, EVFILT_READ);
+}
+
+Scheduler::IOHandler
+Scheduler::install_write_handler (int fd, IOHandler handler, void *closure)
+{
+	Log::log (Log::DEBUG, "install_write_handler (%d)", fd);
+	return Scheduler::install_handler (fd, handler, closure, EVFILT_WRITE);
+}
+
+Scheduler::IOHandler
+Scheduler::remove_write_handler (int fd)
+{
+	Log::log (Log::DEBUG, "remove_write_handler (%d)", fd);
+	return Scheduler::remove_handler (fd, EVFILT_WRITE);
+}
+
+void
+Scheduler::install_recurring_task(const timeval& interval, TimedProc proc, void *closure)
+{
+	timeval			now;
+
+	assert(!recurring_proc);
+	assert(proc);
+	assert(interval.tv_sec >= 0);
+	assert(interval.tv_usec >= 0 && interval.tv_usec < 1000000);
+	assert(interval.tv_sec || interval.tv_usec);
+
+	recurring_proc = proc;
+	recurring_closure = closure;
+	recurring_interval = interval;
+	ntasks++;
+	(void) gettimeofday(&now, NULL);
+	next_task_time = now + interval;
+}
+
+void
+Scheduler::remove_recurring_task(TimedProc proc, void *closure)
+{
+	assert(proc == recurring_proc);
+	assert(closure == recurring_closure);
+
+	recurring_proc = NULL;
+	recurring_closure = closure;
+	timerclear(&recurring_interval);
+	ntasks--;
+}
+
+void
+Scheduler::install_onetime_task (const timeval& when, TimedProc proc, void *closure)
+{
+	onetime_task		**fp = &first_task;
+	onetime_task		*nt;
+
+	while (*fp && (*fp)->when < when)
+		fp  = &(*fp)->next;
+
+	nt = new onetime_task;
+	nt->next = *fp;
+	*fp = nt;
+	nt->when = when;
+	nt->proc = proc;
+	nt->closure = closure;
+	++ntasks;
+}
+
+void
+Scheduler::remove_onetime_task (TimedProc proc, void *closure)
+{
+	onetime_task		*p, **pp = &first_task;
+
+	while ((p = *pp) != NULL) {
+		if (p->proc == proc && p->closure == closure) {
+			*pp = p->next;
+			delete p;	
+			ntasks--;
+			break;
+		}
+		pp = &p->next;
+	}
+}
+
+timeval *
+Scheduler::calc_timeout (void)
+{
+	static timeval		sleep_interval;
+
+	if (ntasks) {
+		timeval		wake_time, now;
+
+		if (recurring_proc)
+			wake_time = next_task_time;
+		if (!recurring_proc || first_task && first_task->when < wake_time)
+			wake_time = first_task->when;
+
+		(void) gettimeofday(&now, NULL);
+
+		sleep_interval = wake_time - now;
+		if (sleep_interval.tv_sec < 0)
+			timerclear(&sleep_interval);
+
+		return (&sleep_interval);
+	} else {
+	        return (NULL);
+	}
+}
+
+//  do_tasks activates all timer based tasks.  It also sets
+//  next_task_time to the absolute time when it should next be
+//  invoked.
+void
+Scheduler::do_tasks()
+{
+	if (ntasks) {
+		timeval		now;
+
+		(void) gettimeofday(&now, NULL);
+
+		if (recurring_proc && now >= next_task_time) {
+			(*recurring_proc)(recurring_closure);
+			next_task_time += recurring_interval;
+
+			if (next_task_time < now)
+				next_task_time = now + recurring_interval;
+	        } else {
+			timeval		time_left = next_task_time - now;
+
+			if (recurring_interval < time_left) {
+				next_task_time = now + recurring_interval;
+			}
+		}
+
+		while (first_task && first_task->when < now) {
+			TimedProc proc = first_task->proc;
+			void *closure = first_task->closure;
+
+			remove_onetime_task(proc, closure);
+			(*proc)(closure);
+		}
+	}
+}
+
+void
+Scheduler::select (void)
+{
+	int			cnt;
+	struct timespec		ts, *_ts;
+	timeval			*timeout = calc_timeout ();
+	struct kevent		*kev;
+
+	if (timeout) {
+		TIMEVAL_TO_TIMESPEC(timeout, &ts);
+		_ts = &ts;
+	} else {
+		_ts = NULL;
+	}
+
+	cnt = ::kevent (kq, new_events, new_cnt, kevents, 50, _ts);
+	if (cnt == -1) {
+		Log::perror ("kevent");
+		::exit (1);
+	}
+	new_cnt = 0;
+
+	for (kev = kevents; cnt; ++kev,--cnt) {
+		handler_data	*data;
+
+		if (kev->flags & EV_ERROR)
+			continue;
+
+		Log::log (Log::DEBUG, "event %d on %d flags %d", kev->filter, kev->ident, kev->flags);
+		data = (handler_data *)kev->udata;
+		data->handler(kev->ident, data->closure);
+	}
+
+	do_tasks();
+}
+#endif /* HAVE_KQUEUE */
