--- fam/Scheduler.h.orig	Tue Jul 16 08:14:41 2002
+++ fam/Scheduler.h	Mon Sep  9 14:16:16 2002
@@ -1,3 +1,4 @@
+//  Copyright (C) 2000 David Rufino <daverufino@btinternet.com>
 //  Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 //  
 //  This program is free software; you can redistribute it and/or modify it
@@ -28,6 +29,14 @@
 
 #include "Boolean.h"
 
+#ifdef HAVE_KQUEUE
+#include <sys/types.h>
+#include <sys/event.h>
+#include <list>
+#include "Log.h"
+#endif
+
+
 //  Scheduler is a class encapsulating a simple event and timer based
 //  task scheduler.  Tasks are simply subroutines that are activated
 //  by calling them.
@@ -87,9 +96,62 @@
     static void exit()			{ running = false; }
     static void loop()			{ running = true;
 					  while (running) select(); }
+#ifdef HAVE_KQUEUE
+    static void init() {
+    kq = ::kqueue();
+    if (kq == -1) {
+	Log::perror ("kqueue");
+		::exit (-2);
+    }
+    ntasks = 0;
+}
+#endif
 
 private:
+#ifdef HAVE_KQUEUE
+
+	static IOHandler		install_handler (int fd, IOHandler handler, void *closure, int filter);
+	static IOHandler		remove_handler (int fd, int filter);
+	static struct kevent *		get_new_event (void);
+	static std::list<struct kevent>::iterator	kev_find (int fd, int filter);
+	static void			do_tasks (void);
+
+	struct handler_data {
+		IOHandler		handler;
+		void			*closure;
+	};
+
+	struct onetime_task {
+		onetime_task		*next;
+		timeval			when;
+		Scheduler::TimedProc	proc;
+		void			*closure;
+	};
+
+	static bool			running;
+
+	// Kqueue stuff
+	static int			kq;
+	static struct kevent		kevents[50];
+	static std::list<struct kevent>	my_list;
+
+	// put new events here 
+	static struct kevent		new_events[10];
+	static int			new_cnt;
+
+	static int			ntasks;
+	static timeval			next_task_time;
+	static onetime_task		*first_task;
+	static TimedProc		recurring_proc;
+	static void			*recurring_closure;
+	static timeval			recurring_interval;
+
+	static timeval *		calc_timeout (void);
 
+	Scheduler();
+
+};
+#else /* HAVE_KQUEUE */
     //  Per-filedescriptor info is the set of three handlers and their
     //  closures.
 
@@ -105,7 +167,11 @@
     //  for the corresponding I/O type.
 
     struct IOTypeInfo {
+#if defined(__FreeBSD__) && defined(__GNUC__) && (__GNUC__ > 2)
+	FDInfo::FDIOHandler FDInfo::*iotype;
+#else
 	FDInfo::FDIOHandler FDInfo::*const iotype;
+#endif
 	unsigned int nbitsset;		// number of bits set in fds
 	fd_set fds;
 	IOTypeInfo(FDInfo::FDIOHandler FDInfo::* a_iotype) :
@@ -154,5 +220,5 @@
     Scheduler();			// Never instantiate a Scheduler.
 
 };
-
+#endif /* HAVE_KQUEUE */
 #endif /* !Scheduler_included */
